@@ -2,9 +2,6 @@ from ehelply_bootstrapper.integrations.integration import Integration
 from ehelply_bootstrapper.utils.state import State
 from ehelply_microservice_library.integrations.fact import get_fact_endpoint
 
-from datetime import datetime
-from typing import Union, List, Dict, Any, Tuple
-
 
 class Places(Integration):
     def __init__(self) -> None:
