@@ -3,11 +3,12 @@ from ehelply_bootstrapper.integrations.integration import Integration
 from ehelply_microservice_library.integrations.fact import get_fact_endpoint
 
 from ehelply_bootstrapper.utils.state import State
+from typing import Union
 
 
 class User(Integration):
     """
-    Note integration is used to talk to the ehelply-notes microservice
+    User integration is used to talk to the ehelply-users microservice
     """
 
     def __init__(self) -> None:
@@ -23,3 +24,12 @@ class User(Integration):
 
     def get_base_url(self) -> str:
         return get_fact_endpoint('ehelply-users')
+
+    def get_participant(self, participant_uuid: str) -> Union[dict, None]:
+        response = self.m2m.requests.get(
+            self.get_base_url() + "/participants/" + participant_uuid)
+
+        if response.status_code != 200:
+            return None
+
+        return response.json()
