@@ -1,44 +1,55 @@
 from typing import List, Union, Tuple
 from pathlib import Path
+import glob
+import os
+import json
+import asyncio
 
 from slugify import slugify
 
-import glob
-
-import os
-
-import json
-
-from ehelply_bootstrapper.bootstrap import Bootstrap
+from ehelply_bootstrapper.bootstrap import Bootstrap, LOADABLE_MYSQL
 from ehelply_bootstrapper.drivers.mysql import MySQLCredentials
+from ehelply_bootstrapper.drivers.aws_utils.aws_asm import ASM
 from ehelply_bootstrapper.utils.state import State
 from ehelply_bootstrapper.utils.environment import Environment
 from ehelply_bootstrapper.utils.service import ServiceMeta, ServiceStatus, KPI
 from ehelply_bootstrapper.utils.cryptography import Hashing
-from ehelply_bootstrapper.bootstrap import LOADABLE_MYSQL
 
 from ehelply_logger.Logger import VERBOSITY_NORMAL
 
 from ehelply_microservice_library.integrations.m2m import M2M
+from ehelply_microservice_library.integrations.sdk import SDK
 from ehelply_microservice_library.integrations.fact import Fact, FactBase, get_fact_endpoint, get_fact_stage
 from ehelply_microservice_library.integrations.logging import Log
 from ehelply_microservice_library.integrations.meta import Meta
 from ehelply_microservice_library.integrations.note import Note
 from ehelply_microservice_library.integrations.monitor import Monitor
 from ehelply_microservice_library.integrations.user import User
-from ehelply_microservice_library.integrations.access import Access, NodeCreate
+from ehelply_microservice_library.integrations.access import Access
 from ehelply_microservice_library.integrations.security import Security
 from ehelply_microservice_library.integrations.appointments import Appointments
 from ehelply_microservice_library.integrations.places import Places
 from ehelply_microservice_library.integrations.reviews import Reviews
+from ehelply_microservice_library.integrations.products import Products
+from ehelply_microservice_library.integrations.notifications import Notifications, NotificationType
 
-from ehelply_microservice_library.utils.constants.constants_time import *
 from ehelply_microservice_library.utils.constants.constants_gatekeeper import *
 from ehelply_microservice_library.utils.service_updater import ServiceUpdater
 from ehelply_microservice_library.utils.secret_names import secret_name_database, secret_name_facts_vault, \
     secret_name_security_vault, secret_name_updater_vault
 
-from ehelply_microservice_library.aws.aws_asm import ASM
+from ehelply_microservice_library.utils.ehelply_cloud import AuthDynamos
+
+CONST_INTEGRATION_ACCESS: str = "access"
+CONST_INTEGRATION_APPOINTMENTS: str = "appointments"
+CONST_INTEGRATION_META: str = "meta"
+CONST_INTEGRATION_NOTES: str = "notes"
+CONST_INTEGRATION_NOTIFICATIONS: str = "notifications"
+CONST_INTEGRATION_PLACES: str = "places"
+CONST_INTEGRATION_REVIEWS: str = "reviews"
+CONST_INTEGRATION_SECURITY: str = "security"
+CONST_INTEGRATION_USERS: str = "users"
+CONST_INTEGRATION_PRODUCTS: str = "products"
 
 
 class ServiceBootstrap(Bootstrap):
@@ -136,6 +147,27 @@ class ServiceBootstrap(Bootstrap):
         raise Exception(
             "get_service_loadables must be overridden in your service file. Please see `service.py`")
 
+    def get_service_required_integrations(self) -> List[str]:
+        """
+        This defines which integrations are required.
+
+        Returns:
+            List[str]: A list of integrations
+
+        """
+        raise Exception(
+            "get_service_required_integrations must be overridden in your service file. Please see `service.py`")
+
+    def get_service_optional_integrations(self) -> List[str]:
+        """
+        This defines which integrations are optional.
+
+        Returns:
+            List[str]: A list of integrations
+
+        """
+        return []
+
     def get_service_summary(self) -> str:
         """
         A summary of the service
@@ -180,6 +212,16 @@ class ServiceBootstrap(Bootstrap):
         """
         return json.loads(os.environ[GATEKEEPER_KEY_ENVIRONMENT_NAME])['secret_key']
 
+    def get_root_path(self) -> Path:
+        """
+        Gets the root path
+
+        Returns:
+            Path: The root path
+
+        """
+        raise Exception("get_root_path must be overridden in service.py to be used.")
+
     def get_https_certificate_paths(self) -> Tuple[Union[None, str], Union[None, str]]:
         """
         Gets the certificates to run the dev server as https
@@ -248,10 +290,10 @@ class ServiceBootstrap(Bootstrap):
 
     def get_service_package_path(self) -> Path:
         """
-        This is an override for the package path
+        This is an override for the microservice path
 
         Returns:
-            str: Path of ehelply-package.json file
+            str: Path of the microservice root
 
         """
         return None
@@ -276,13 +318,12 @@ class ServiceBootstrap(Bootstrap):
         """
         return []
 
-    def get_access_nodes(self) -> List[NodeCreate]:
+    def get_notification_types(self) -> List[NotificationType]:
         """
-        List of permission nodes to push when the application loads
-        You cannot use permissions for this microservice which are not defined here
+        List of notification types to push when the microservice loads
 
         Returns:
-            List[NodeCreate]: Nodes to be used throughout this service
+            List[NotificationType]: Notification types to be used throughout this service
 
         """
         return []
@@ -297,6 +338,14 @@ class ServiceBootstrap(Bootstrap):
         """
         return []
 
+    def is_load_service_routers(self) -> bool:
+        """
+        Whether or not to register the service routers or not
+        Returns:
+            Bool
+        """
+        return True
+
     def fastapi_routers(self):
         """
         Register fastapi routers
@@ -308,37 +357,39 @@ class ServiceBootstrap(Bootstrap):
 
         # !!!!! Imports for routers must go here to remove an order of operation bug that occurs when endpoints need to
         #          talk to a DB
-        from ehelply_microservice_library.routers import router_service_monitor
-        self.fastapi_driver.instance.include_router(
-            router_service_monitor.router,
-            prefix="/ehelply_monitor",
-            tags=["ehelply", "service"],
-            responses={404: {"description": "Not found"}},
-        )
 
-        from ehelply_microservice_library.routers import router_service_meta as service_meta_router
-        self.fastapi_driver.instance.include_router(
-            service_meta_router.router,
-            prefix="/ehelply_meta",
-            tags=["ehelply", "service"],
-            responses={404: {"description": "Not found"}},
-        )
+        if self.is_load_service_routers():
+            from ehelply_microservice_library.routers import router_service_monitor
+            self.fastapi_driver.instance.include_router(
+                router_service_monitor.router,
+                prefix="/ehelply_monitor",
+                tags=["ehelply", "service"],
+                responses={404: {"description": "Not found"}},
+            )
 
-        from ehelply_microservice_library.routers import router_service as service_router
-        self.fastapi_driver.instance.include_router(
-            service_router.router,
-            prefix="/ehelply_service",
-            tags=["ehelply", "service"],
-            responses={404: {"description": "Not found"}},
-        )
+            from ehelply_microservice_library.routers import router_service_meta as service_meta_router
+            self.fastapi_driver.instance.include_router(
+                service_meta_router.router,
+                prefix="/ehelply_meta",
+                tags=["ehelply", "service"],
+                responses={404: {"description": "Not found"}},
+            )
 
-        from ehelply_microservice_library.routers import router_service_security as service_security_router
-        self.fastapi_driver.instance.include_router(
-            service_security_router.router,
-            prefix="/ehelply_security",
-            tags=["ehelply", "service"],
-            responses={404: {"description": "Not found"}},
-        )
+            from ehelply_microservice_library.routers import router_service as service_router
+            self.fastapi_driver.instance.include_router(
+                service_router.router,
+                prefix="/ehelply_service",
+                tags=["ehelply", "service"],
+                responses={404: {"description": "Not found"}},
+            )
+
+            from ehelply_microservice_library.routers import router_service_security as service_security_router
+            self.fastapi_driver.instance.include_router(
+                service_security_router.router,
+                prefix="/ehelply_security",
+                tags=["ehelply", "service"],
+                responses={404: {"description": "Not found"}},
+            )
 
     def get_secret_subscriptions(self) -> List[str]:
         """
@@ -391,39 +442,92 @@ class ServiceBootstrap(Bootstrap):
 
                                        "mq-logging",
                                        "mq-monitoring",
+                                       "mq-usage",
 
-                                       "ehelply-access",
-                                       "ehelply-security",
-                                       "ehelply-notes",
-                                       "ehelply-meta",
-                                       "ehelply-monitors",
-                                       "ehelply-users",
-                                       "ehelply-access",
                                        "ehelply-templates",
-                                       # "ehelply-places", TODO: Uncomment once added into facts seeding
-                                       # "ehelply-reviews", TODO: Uncomment once added into facts seeding
-                                       # "ehelply-appointments", TODO: Uncomment once added into facts seeding
-
+                                       "ehelply-sam"
                                    ] + self.get_fact_subscriptions()
 
+        active_integrations: List[
+            str] = self.get_service_required_integrations() + self.get_service_optional_integrations()
+
+        if CONST_INTEGRATION_ACCESS in active_integrations:
+            subscribe_to_facts.append("ehelply-access")
+
+        if CONST_INTEGRATION_SECURITY in active_integrations:
+            subscribe_to_facts.append("ehelply-security")
+
+        if CONST_INTEGRATION_NOTES in active_integrations:
+            subscribe_to_facts.append("ehelply-notes")
+
+        if CONST_INTEGRATION_META in active_integrations:
+            subscribe_to_facts.append("ehelply-meta")
+
+        if CONST_INTEGRATION_USERS in active_integrations:
+            subscribe_to_facts.append("ehelply-users")
+
+        if CONST_INTEGRATION_PRODUCTS in active_integrations:
+            subscribe_to_facts.append("ehelply-products")
+
+        if CONST_INTEGRATION_NOTIFICATIONS in active_integrations:
+            subscribe_to_facts.append("mq-notifications")
+            subscribe_to_facts.append("ehelply-notifications")
+
+        if CONST_INTEGRATION_PLACES in active_integrations:
+            pass
+            # TODO: Once added to facts: subscribe_to_facts.append("ehelply-places")
+
+        if CONST_INTEGRATION_REVIEWS in active_integrations:
+            pass
+            # TODO: Once added to facts: subscribe_to_facts.append("ehelply-reviews")
+
+        if CONST_INTEGRATION_APPOINTMENTS in active_integrations:
+            pass
+            # TODO: Once added to facts: subscribe_to_facts.append("ehelply-appointments")
+
         # Register the fact service integration
         self.register_integration(Fact(State.logger, subscribe_to_facts))
 
         # Register the M2M service integration
         self.register_integration(M2M(self.service_gatekeeper_key))
 
-        # Register the service integrations
-        self.register_integration(
-            Access(self.service_meta.name, self.service_meta.summary, self.get_access_nodes()))
-        self.register_integration(Security())
+        # Register the SDK service integration
+        self.register_integration(SDK(self.service_gatekeeper_key))
+
+        # Register the core service integration
         self.register_integration(Log(self.service_meta.key))
-        self.register_integration(Note())
-        self.register_integration(Meta())
         self.register_integration(Monitor())
-        self.register_integration(User())
-        self.register_integration(Appointments())
-        self.register_integration(Places())
-        self.register_integration(Reviews())
+
+        # Register the service integrations
+        if CONST_INTEGRATION_ACCESS in active_integrations:
+            self.register_integration(Access())
+
+        if CONST_INTEGRATION_SECURITY in active_integrations:
+            self.register_integration(Security())
+
+        if CONST_INTEGRATION_NOTES in active_integrations:
+            self.register_integration(Note())
+
+        if CONST_INTEGRATION_META in active_integrations:
+            self.register_integration(Meta())
+
+        if CONST_INTEGRATION_USERS in active_integrations:
+            self.register_integration(User())
+
+        if CONST_INTEGRATION_NOTIFICATIONS in active_integrations:
+            self.register_integration(Notifications(self.get_notification_types()))
+
+        if CONST_INTEGRATION_PLACES in active_integrations:
+            self.register_integration(Places())
+
+        if CONST_INTEGRATION_REVIEWS in active_integrations:
+            self.register_integration(Reviews())
+
+        if CONST_INTEGRATION_APPOINTMENTS in active_integrations:
+            self.register_integration(Appointments())
+
+        if CONST_INTEGRATION_PRODUCTS in active_integrations:
+            self.register_integration(Products())
 
     def get_mysql_credentials(self) -> MySQLCredentials:
         """
@@ -474,7 +578,7 @@ class ServiceBootstrap(Bootstrap):
             ]
             alembic.config.main(argv=alembicArgs)
 
-    def monitor(self):
+    async def monitor(self):
         """
         Creates link to a monitor service via a new thread
 
@@ -483,7 +587,7 @@ class ServiceBootstrap(Bootstrap):
 
         """
         monitor_integration: Monitor = State.integrations.get("monitor")
-        monitor_integration.register_service()
+        # monitor_integration.register_service()
 
         State.logger.info("Booting async monitor... Expect to see out of order Batcher info in the logs.")
         from ehelply_microservice_library.routers.router_service_monitor import MonitorHeartbeatThread
@@ -522,7 +626,7 @@ class ServiceBootstrap(Bootstrap):
         except:
             State.logger.warning("Unable to check for service template updates")
 
-    def fetch_keys(self):
+    async def fetch_keys(self):
         """
         Fetches keys from the security microservice
 
@@ -531,7 +635,7 @@ class ServiceBootstrap(Bootstrap):
 
         """
         from ehelply_microservice_library.routers.router_service_security import refresh_encryption_keys
-        refresh_encryption_keys(category=self.service_meta.key)
+        await refresh_encryption_keys(category=self.service_meta.key)
 
         State.logger.info("Booting security key fetcher... Expect to see out of order Batcher/Timer info in the logs.")
         from ehelply_microservice_library.routers.router_service_security import SecurityKeysThread
@@ -542,6 +646,23 @@ class ServiceBootstrap(Bootstrap):
         )
         security_keys_thread.start()
 
+    async def fetch_kpis(self):
+        """
+        Fetches KPIs from this microservice. Look into bootstrapper.get_kpi()
+
+        Returns:
+            None
+
+        """
+        State.logger.info("Booting service KPI fetcher... Expect to see out of order Batcher/Timer info in the logs.")
+        from ehelply_microservice_library.routers.router_service_monitor import KPIFetcherThread
+        kpi_fetcher_thread = KPIFetcherThread(
+            logger=State.logger,
+            queue_url=get_fact_endpoint("mq-monitoring"),
+            service_meta=self.service_meta
+        )
+        kpi_fetcher_thread.start()
+
     def configure_security(self):
         """
         Hashing cost factor is a function of 2^COST_FACTOR
@@ -564,7 +685,7 @@ class ServiceBootstrap(Bootstrap):
 
         self.configure_security()
 
-    def before_integrations(self):
+    async def before_integrations(self):
         """
         EVENT HOOK - Runs before microservice loads integrations
 
@@ -576,7 +697,7 @@ class ServiceBootstrap(Bootstrap):
 
         self.fetch_secrets()
 
-    def before_drivers(self):
+    async def before_drivers(self):
         """
         EVENT HOOK - Runs before microservice loads drivers
 
@@ -590,9 +711,9 @@ class ServiceBootstrap(Bootstrap):
 
         State.logger.info("Fetching encryption keys...")
 
-        self.fetch_keys()
+        await self.fetch_keys()
 
-    def post_load(self):
+    async def post_load(self):
         """
         Entry point of the application that is executed after the bootstrapping process completes
 
@@ -603,10 +724,17 @@ class ServiceBootstrap(Bootstrap):
             None
 
         """
+        State.logger.info("Setting up Dynamo auth")
+
+        AuthDynamos.make()
 
         State.logger.info("Integrating into cluster...")
 
-        self.monitor()
+        await self.monitor()
+
+        State.logger.info("Setting up KPI fetching...")
+
+        await self.fetch_kpis()
 
         State.logger.info("Checking for service template updates...")
 
