@@ -4,6 +4,9 @@ from ehelply_microservice_library.integrations.fact import get_fact_endpoint
 
 
 class ServiceUpdater:
+    """
+    Wrapper class for ehelply_updater
+    """
     def __init__(self, access_key: str, secret_key: str, root_path: Path) -> None:
         self.config: Config = Config(
             api_base_url=get_fact_endpoint('ehelply-templates') + '/templates',
